// Make React available globally
import React from "react";
window.React = React;

// Import other dependencies
import ReactDOM from "react-dom/client";
import App from "./App.tsx";
import "./index.css";
import { BrowserRouter } from "react-router-dom";
import { HelmetProvider } from "react-helmet-async";
import { AuthProvider } from "./contexts/AuthContext";
import { ClerkProvider } from "@clerk/clerk-react";
import { monitorWebVitals } from './utils/performanceOptimizations'

// Force useLayoutEffect to be useEffect in SSR
if (typeof window === 'undefined') {
  React.useLayoutEffect = React.useEffect;
}

// Get the Clerk publishable key
const clerkPubKey = import.meta.env.VITE_CLERK_PUBLISHABLE_KEY || '';

// Initialize root element
const root = document.getElementById("root");
if (!root) {
  throw new Error("Root element not found");
}

// Inject React into window just to be extra sure
window.React = window.React || React;

// Initialize performance monitoring
if (process.env.NODE_ENV === 'production') {
  monitorWebVitals();
}

// Define an app container with performance optimizations
const AppContainer = () => (
  <React.StrictMode>
    <ClerkProvider 
      publishableKey={clerkPubKey}
      appearance={{
        variables: {
          colorPrimary: "black",
          borderRadius: "0.375rem",
        },
        elements: {
          formButtonPrimary: 
            "bg-black hover:bg-black-800 text-white py-2 px-4 rounded-md",
          card: "bg-white shadow-xl border border-gray-200 rounded-lg",
          logoBox: "hidden",
          logoImage: "hidden",
          header: "text-xl text-center font-bold mb-4",
          socialButtonsBlockButton: 
            "border border-gray-300 bg-white hover:bg-gray-50 text-black",
          formFieldLabel: "text-gray-700",
          formFieldInput: 
            "bg-white border border-gray-300 rounded-md focus:ring-2 focus:ring-black focus:border-transparent",
          dividerLine: "bg-gray-200",
          dividerText: "text-gray-500 mx-3",
          footerActionLink: "text-black hover:text-gray-600 underline",
          identityPreviewEditButtonIcon: "text-black",
          alert: "rounded-lg",
        }
      }}
      localization={{
        signIn: {
          phoneCode: {
            title: "Sign in to Nynexa Foundation",
          },
          start: {
            title: "Sign in to Nynexa Foundation",
            subtitle: "to continue to our platform",
          }
        }
      }}
      afterSignInUrl="/"
      afterSignUpUrl="/"
    >
      <HelmetProvider>
        <BrowserRouter>
          <AuthProvider>
            <App />
          </AuthProvider>
        </BrowserRouter>
      </HelmetProvider>
    </ClerkProvider>
  </React.StrictMode>
);

// Use the fastest method to render
ReactDOM.createRoot(root).render(<AppContainer />);

// Create a load event after initial render to mark when app is ready
window.addEventListener('load', () => {
  // Add a class to body once loaded
  document.body.classList.add('app-loaded');
  
  // Register Service Worker if available
  if ('serviceWorker' in navigator && process.env.NODE_ENV === 'production') {
    window.addEventListener('load', () => {
      navigator.serviceWorker.register('/service-worker.js')
        .then(registration => {
          console.log('SW registered:', registration);
        })
        .catch(error => {
          console.log('SW registration failed:', error);
        });
    });
  }
});

// Use modern idle callback to defer non-critical work
if ('requestIdleCallback' in window) {
  (window as any).requestIdleCallback(() => {
    // Initialize optional features after main content is loaded
    import('./utils/optionalFeatures')
      .then(module => {
        const initOptionalFeatures = module.default;
        initOptionalFeatures();
      })
      .catch(err => {
        console.warn('Failed to load optional features:', err);
      });
  });
} else {
  // Fallback for browsers that don't support requestIdleCallback
  setTimeout(() => {
    import('./utils/optionalFeatures')
      .then(module => {
        const initOptionalFeatures = module.default;
        initOptionalFeatures();
      })
      .catch(err => {
        console.warn('Failed to load optional features:', err);
      });
  }, 3000);
}
